IF EXISTS (SELECT * FROM sys.objects WHERE type = 'P' AND name = 'FW_MigrateCatalog')
DROP PROCEDURE FW_MigrateCatalog
GO

Create Procedure FW_MigrateCatalog
AS
BEGIN

MERGE FW_ExecutableMT AS T
USING HIP8_Executable AS S
ON (T.id = S.id)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (id,version,lastModifyingUsername,lastModified,name,description,filename,fingerprint,signerName,note, TenantId) VALUES (S.id,S.version, S.lastModifyingUsername,S.lastModified,S.name,S.description,S.filename,S.fingerprint,S.signerName,S.note,'1')
WHEN MATCHED
	THEN UPDATE SET T.id = S.id ,T.version = S.version , T.lastModifyingUsername = S.lastModifyingUsername, T.lastModified = S.lastModified ,T.name = S.name ,T.description = S.description, T.filename = S.filename, T.fingerprint = S.fingerprint, T.signerName = S.signerName, T.note = S.note, T.TenantId = '1';

MERGE FW_ApplicationMT AS T
USING HIP8_Application AS S
ON (T.id = S.id)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (id,version,lastModifyingUsername,lastModified,name,note, TenantId) VALUES (S.id,S.version, S.lastModifyingUsername,S.lastModified, S.name, S.note,'1')
WHEN MATCHED
	THEN UPDATE SET T.id = S.id ,T.version = S.version , T.lastModifyingUsername = S.lastModifyingUsername, T.lastModified = S.lastModified ,T.name = S.name , T.note = S.note, T.TenantId = '1';

MERGE FW_RuleMT AS T
USING HIP8_Rule AS S
ON (T.id = S.id)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (id, version, lastModifyingUsername, lastModified, name,note, enabled, action, direction, mediaFlags, tcpFlagsFlags, transportProtocol, localServiceList, remoteServiceList, intrusion, trafficLogged, schedule_start, schedule_end, schedule_offHours, schedule_weekMask, schedule_clickTimeout, DTYPE,leafNodeId, TenantId,schedule_startHours,schedule_startMinutes,schedule_endHours,schedule_endMinutes,scheduleEnabled) VALUES (S.id, S.version, S.lastModifyingUsername, S.lastModified, S.name, S.note, CASE WHEN S.schedule_offHours!='NONE' AND S.schedule_weekMask='0' THEN '0' ELSE S.enabled END, S.action, S.direction, S.mediaFlags, S.tcpFlagsFlags, S.transportProtocol, S.localServiceList, S.remoteServiceList, S.intrusion,   S.trafficLogged,S.schedule_start,S.schedule_end,S.schedule_offHours, S.schedule_weekMask, S.schedule_clickTimeout, S.DTYPE,S.leafNodeId,'1',PARSENAME(REPLACE(S.schedule_start,':','.'),2),PARSENAME(REPLACE(S.schedule_start,':','.'),1),PARSENAME(REPLACE(S.schedule_end,':','.'),2),PARSENAME(REPLACE(S.schedule_end,':','.'),1),CASE WHEN S.schedule_weekMask='0' THEN '0' ELSE '1' END)
WHEN MATCHED
	THEN UPDATE SET T.id = S.id ,T.version = S.version , T.lastModifyingUsername = S.lastModifyingUsername, T.lastModified = S.lastModified ,T.name = S.name , T.note = S.note,T.action=S.action,T.direction = S.direction, T.mediaFlags = S.mediaFlags, T.tcpFlagsFlags = S.tcpFlagsFlags, T.transportProtocol = S.transportProtocol, T.localServiceList = S.localServiceList, T.remoteServiceList = S.remoteServiceList, T.intrusion= S.intrusion, T.trafficLogged = S.trafficLogged, T.schedule_start = S.schedule_start,T.schedule_end= S.schedule_end,T.schedule_offHours = S.schedule_offHours, T.schedule_weekMask = S.schedule_weekMask,T.schedule_clickTimeout = S.schedule_clickTimeout,T.DTYPE = S.DTYPE, T.leafNodeId = S.leafNodeId, T.TenantId = '1' ,T.schedule_startHours=PARSENAME(REPLACE(S.schedule_start,':','.'),2) ,T.schedule_startMinutes=PARSENAME(REPLACE(S.schedule_start,':','.'),1) ,T.schedule_endHours=PARSENAME(REPLACE(S.schedule_end,':','.'),2),T.schedule_endMinutes=PARSENAME(REPLACE(S.schedule_end,':','.'),1),T.scheduleEnabled=CASE WHEN S.schedule_weekMask='0' THEN '0' ELSE '1' END,T.enabled=CASE WHEN S.schedule_offHours!='NONE' AND S.schedule_weekMask='0' THEN '0' ELSE S.enabled END;


MERGE FW_Rule_ApplicationMT AS T
USING HIP8_Rule_Application AS S
ON (T.RuleId = S.RuleId) AND (T.ApplicationId = S.ApplicationId)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (RuleId, ApplicationId, TenantId) VALUES (S.RuleId, S.ApplicationId,'1')
WHEN MATCHED
	THEN UPDATE SET T.RuleId = S.RuleId ,T.ApplicationId = S.ApplicationId, T.TenantId = '1';

MERGE FW_NamedNetworkMT AS T
USING HIP8_NamedNetwork AS S
ON (T.id = S.id)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (id,version,lastModifyingUsername,lastModified,name,note, TenantId) VALUES (S.id,S.version, S.lastModifyingUsername,S.lastModified,S.name,S.note,'1')
WHEN MATCHED
	THEN UPDATE SET T.id = S.id ,T.version = S.version , T.lastModifyingUsername = S.lastModifyingUsername, T.lastModified = S.lastModified ,T.name = S.name ,T.note = S.note, T.TenantId = '1';

MERGE FW_NamedNetwork_HostsMT AS T
USING HIP8_NamedNetwork_Hosts AS S
ON (T.NamedNetworkId = S.NamedNetworkId) AND (T.address = S.address)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (NamedNetworkId, address, TenantId) VALUES (S.NamedNetworkId, S.address,'1')
WHEN MATCHED
	THEN UPDATE SET T.NamedNetworkId = S.NamedNetworkId ,T.address = S.address, T.TenantId = '1';

--Delete child entries in the FW catalog not in the HIPS catalog if the parent (NamedNetworkId) is in both.
--The parent in both check excludes parents created in the FW catalog but are not in the HIPS catalog.  These parents
--could have been introduced by the customer or previous migrations, we can't tell which one.
DELETE FW_NamedNetwork_HostsMT FROM FW_NamedNetwork_HostsMT AS FW
LEFT JOIN HIP8_NamedNetwork_Hosts AS HIP
	ON FW.NamedNetworkId = HIP.NamedNetworkId AND FW.address = HIP.address
WHERE HIP.NamedNetworkId IS NULL
	AND EXISTS (SELECT 1 FROM HIP8_NamedNetwork_Hosts AS HIP2 WHERE HIP2.NamedNetworkId = FW.NamedNetworkId)

MERGE FW_Rule_LocalNetworkMT AS T
USING HIP8_Rule_LocalNetwork AS S
ON (T.RuleId = S.RuleId) AND (T.NamedNetworkId = S.NamedNetworkId)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (RuleId, NamedNetworkId, TenantId) VALUES (S.RuleId, S.NamedNetworkId,'1')
WHEN MATCHED
	THEN UPDATE SET T.RuleId = S.RuleId, T.NamedNetworkId = S.NamedNetworkId , T.TenantId = '1';

MERGE FW_Rule_RemoteNetworkMT AS T
USING HIP8_Rule_RemoteNetwork AS S
ON (T.RuleId = S.RuleId) AND (T.NamedNetworkId = S.NamedNetworkId)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (RuleId, NamedNetworkId, TenantId) VALUES (S.RuleId, S.NamedNetworkId,'1')
WHEN MATCHED
	THEN UPDATE SET T.RuleId = S.RuleId, T.NamedNetworkId = S.NamedNetworkId , T.TenantId = '1';

MERGE FW_Rule_NetworkProtocolMT AS T
USING HIP8_Rule_NetworkProtocol AS S
ON (T.RuleId = S.RuleId) AND (T.networkProtocol = S.networkProtocol)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (RuleId, networkProtocol, TenantId) VALUES (S.RuleId, S.networkProtocol,'1')
WHEN MATCHED
	THEN UPDATE SET T.RuleId = S.RuleId, T.networkProtocol = S.networkProtocol , T.TenantId = '1';

MERGE FW_Group_RuleMT AS T
USING
(
	-- BZ 1192753: Use group by clause and only merge the 1st rule --
	select [GroupId], [RuleId], min([ChildIndex])
		from [HIP8_Group_Rule]
		group by [GroupId], [RuleId]
) S([GroupId], [RuleId], [ChildIndex])
ON (T.GroupId = S.GroupId) AND (T.RuleId = S.RuleId)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (GroupId, RuleId, ChildIndex, TenantId) VALUES (S.GroupId, S.RuleId, S.ChildIndex, '1')
WHEN MATCHED
	THEN UPDATE SET T.ChildIndex = S.ChildIndex;

MERGE FW_ServiceNameMT AS T
USING HIP8_ServiceName AS S
ON (T.id = S.id)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (id,version,lastModifyingUsername,lastModified,name,note, ipProtocol, port, TenantId) VALUES (S.id,S.version, S.lastModifyingUsername,S.lastModified,S.name,S.note, S.ipProtocol, S.port,'1')
WHEN MATCHED
	THEN UPDATE SET T.id = S.id ,T.version = S.version , T.lastModifyingUsername = S.lastModifyingUsername, T.lastModified = S.lastModified ,T.name = S.name ,T.note = S.note, T.ipProtocol = S.ipProtocol, T.port = S.port, T.TenantId = '1';


MERGE FW_ApplicationExecutableMT AS T
USING
(
	-- BZ 1199695:  Use group by clause to prevent merging duplicates. --
	select [ApplicationId], [ExecutableId]
		from [HIP8_ApplicationExecutable]
		group by [ApplicationId], [ExecutableId]
) S([ApplicationId], [ExecutableId])
ON (T.ApplicationId = S.ApplicationId) AND (T.ExecutableId = S.ExecutableId)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (ApplicationId, ExecutableId, TenantId) VALUES (S.ApplicationId, S.ExecutableId,'1');
-- Don't update if matched.  Ids already match, and there's no data to update

--Delete child entries in the FW catalog not in the HIPS catalog if the parent (ApplicationId) is in both.
--The parent in both check excludes parents created in the FW catalog but are not in the HIPS catalog.  These parents
--could have been introduced by the customer or previous migrations, we can't tell which one.
DELETE FW_ApplicationExecutableMT FROM FW_ApplicationExecutableMT AS FW
LEFT JOIN HIP8_ApplicationExecutable AS HIP
	ON FW.ApplicationId = HIP.ApplicationId AND FW.ExecutableId = HIP.ExecutableId
WHERE HIP.ApplicationId IS NULL
	AND EXISTS (SELECT 1 FROM HIP8_ApplicationExecutable AS HIP2 WHERE HIP2.ApplicationId = FW.ApplicationId)

MERGE FW_LocationMT AS T
USING HIP8_Location AS S
ON (T.id = S.id)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (id,version, lastModifyingUsername, lastModified, name, note, isolated, requireEpoReachable, TenantId) VALUES (S.id,S.version, S.lastModifyingUsername,S.lastModified,S.name,S.note,S.isolated, S.requireEpoReachable, '1')
WHEN MATCHED
	THEN UPDATE SET T.id = S.id ,T.version = S.version , T.lastModifyingUsername = S.lastModifyingUsername, T.lastModified = S.lastModified ,T.name = S.name ,T.note = S.note, T.isolated = S.isolated, T.requireEpoReachable = S.requireEpoReachable, T.TenantId = '1';

MERGE FW_Rule_LocationMT AS T
USING HIP8_Rule_Location AS S
ON (T.RuleId = S.RuleId) AND (T.LocationId = S.LocationId)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (RuleId, LocationId, TenantId) VALUES (S.RuleId, S.LocationId,'1')
WHEN MATCHED
	THEN UPDATE SET T.RuleId = S.RuleId, T.LocationId = S.LocationId, T.TenantId = '1';

MERGE FW_Location_DefaultGatewayMT AS T
USING HIP8_Location_DefaultGateway AS S
ON (T.LocationId = S.LocationId) AND (T.address = S.address)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (LocationId, address, TenantId) VALUES (S.LocationId, S.address, '1')
WHEN MATCHED
	THEN UPDATE SET T.LocationId = S.LocationId, T.address = S.address, T.TenantId = '1';

--Delete child entries in the FW catalog not in the HIPS catalog if the parent (LocationId) is in both.
--The parent in both check excludes parents created in the FW catalog but are not in the HIPS catalog.  These parents
--could have been introduced by the customer or previous migrations, we can't tell which one.
DELETE FW_Location_DefaultGatewayMT FROM FW_Location_DefaultGatewayMT AS FW
LEFT JOIN HIP8_Location_DefaultGateway AS HIP
	ON FW.LocationId = HIP.LocationId AND FW.address = HIP.address
WHERE HIP.LocationId IS NULL
	AND EXISTS (SELECT 1 FROM HIP8_Location_DefaultGateway AS HIP2 WHERE HIP2.LocationId = FW.LocationId)

MERGE FW_Location_DhcpServerMT AS T
USING HIP8_Location_DhcpServer AS S
ON (T.LocationId = S.LocationId) AND (T.address = S.address)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (LocationId, address, TenantId) VALUES (S.LocationId, S.address, '1')
WHEN MATCHED
	THEN UPDATE SET T.LocationId = S.LocationId, T.address = S.address, T.TenantId = '1';

DELETE FW_Location_DhcpServerMT FROM FW_Location_DhcpServerMT AS FW
LEFT JOIN HIP8_Location_DhcpServer AS HIP
	ON FW.LocationId = HIP.LocationId AND FW.address = HIP.address
WHERE HIP.LocationId IS NULL
	AND EXISTS (SELECT 1 FROM HIP8_Location_DhcpServer AS HIP2 WHERE HIP2.LocationId = FW.LocationId)

MERGE FW_Location_DnsServerMT AS T
USING HIP8_Location_DnsServer AS S
ON (T.LocationId = S.LocationId) AND (T.address = S.address)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (LocationId, address, TenantId) VALUES (S.LocationId, S.address, '1')
WHEN MATCHED
	THEN UPDATE SET T.LocationId = S.LocationId, T.address = S.address, T.TenantId = '1';

DELETE FW_Location_DnsServerMT FROM FW_Location_DnsServerMT AS FW
LEFT JOIN HIP8_Location_DnsServer AS HIP
	ON FW.LocationId = HIP.LocationId AND FW.address = HIP.address
WHERE HIP.LocationId IS NULL
	AND EXISTS (SELECT 1 FROM HIP8_Location_DnsServer AS HIP2 WHERE HIP2.LocationId = FW.LocationId)

MERGE FW_Location_DnsSuffixMT AS T
USING HIP8_Location_DnsSuffix AS S
ON (T.LocationId = S.LocationId) AND (T.dnsSuffix = S.dnsSuffix)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (LocationId, dnsSuffix, TenantId) VALUES (S.LocationId, S.dnsSuffix, '1')
WHEN MATCHED
	THEN UPDATE SET T.LocationId = S.LocationId, T.dnsSuffix = S.dnsSuffix, T.TenantId = '1';

DELETE FW_Location_DnsSuffixMT FROM FW_Location_DnsSuffixMT AS FW
LEFT JOIN HIP8_Location_DnsSuffix AS HIP
	ON FW.LocationId = HIP.LocationId AND FW.dnsSuffix = HIP.dnsSuffix
WHERE HIP.LocationId IS NULL
	AND EXISTS (SELECT 1 FROM HIP8_Location_DnsSuffix AS HIP2 WHERE HIP2.LocationId = FW.LocationId)

MERGE FW_Location_PrimaryWinsMT AS T
USING HIP8_Location_PrimaryWins AS S
ON (T.LocationId = S.LocationId) AND (T.address = S.address)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (LocationId, address, TenantId) VALUES (S.LocationId, S.address, '1')
WHEN MATCHED
	THEN UPDATE SET T.LocationId = S.LocationId, T.address = S.address, T.TenantId = '1';

DELETE FW_Location_PrimaryWinsMT FROM FW_Location_PrimaryWinsMT AS FW
LEFT JOIN HIP8_Location_PrimaryWins AS HIP
	ON FW.LocationId = HIP.LocationId AND FW.address = HIP.address
WHERE HIP.LocationId IS NULL
	AND EXISTS (SELECT 1 FROM HIP8_Location_PrimaryWins AS HIP2 WHERE HIP2.LocationId = FW.LocationId)

MERGE FW_Location_SecondaryWinsMT AS T
USING HIP8_Location_SecondaryWins AS S
ON (T.LocationId = S.LocationId) AND (T.address = S.address)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (LocationId, address, TenantId) VALUES (S.LocationId, S.address, '1')
WHEN MATCHED
	THEN UPDATE SET T.LocationId = S.LocationId, T.address = S.address, T.TenantId = '1';

DELETE FW_Location_SecondaryWinsMT FROM FW_Location_SecondaryWinsMT AS FW
LEFT JOIN HIP8_Location_SecondaryWins AS HIP
	ON FW.LocationId = HIP.LocationId AND FW.address = HIP.address
WHERE HIP.LocationId IS NULL
	AND EXISTS (SELECT 1 FROM HIP8_Location_SecondaryWins AS HIP2 WHERE HIP2.LocationId = FW.LocationId)

MERGE FW_Location_RegKeyMT AS T
USING HIP8_Location_RegKey AS S
ON (T.LocationId = S.LocationId)
WHEN NOT MATCHED BY TARGET
	THEN INSERT (LocationId,regKey,TenantId) VALUES (S.LocationId,S.regKey,'1')
WHEN MATCHED
	THEN UPDATE SET T.LocationId = S.LocationId,T.regKey = S.regKey,T.TenantId = '1';

DELETE FW_Location_RegKeyMT FROM FW_Location_RegKeyMT AS FW
LEFT JOIN HIP8_Location_RegKey AS HIP
	ON FW.LocationId = HIP.LocationId AND FW.regKey = HIP.regKey
WHERE HIP.LocationId IS NULL
	AND EXISTS (SELECT 1 FROM HIP8_Location_RegKey AS HIP2 WHERE HIP2.LocationId = FW.LocationId)


	--Code Block for Reverse Action Logic

	/*
		If Reverse Action during off hours is enabled in source (Hips Side) two  rules will be created at target (ENS Fw)
		the first rule is identical but with disable during off hours, and the 2nd rule is a copy, with action reversed, no schedule , directly beneath the first rule
	  These  new rules are created from [dbo].ENSMIG_FWMigrationNewRules table from here it is inserted to all rules related tables.
		*/


    --Delete all existing reverse action GUID's if any ,this is for Re migrations

   DELETE R FROM FW_RuleMT R INNER JOIN ENSMIG_FWMigrationNewRules M ON R.id = M.id
   DELETE RA FROM FW_Rule_ApplicationMT RA INNER JOIN ENSMIG_FWMigrationNewRules M ON RA.RuleId = M.id
   DELETE RLNT FROM FW_Rule_LocalNetworkMT RLNT INNER JOIN ENSMIG_FWMigrationNewRules M ON RLNT.RuleId = M.id
   DELETE RRNT FROM FW_Rule_RemoteNetworkMT RRNT INNER JOIN ENSMIG_FWMigrationNewRules M ON RRNT.RuleId = M.id
   DELETE RNTP FROM FW_Rule_NetworkProtocolMT RNTP INNER JOIN ENSMIG_FWMigrationNewRules M ON RNTP.RuleId = M.id
   DELETE RLOC FROM FW_Rule_LocationMT RLOC INNER JOIN ENSMIG_FWMigrationNewRules M ON RLOC.RuleId = M.id
   DELETE GR FROM FW_Group_RuleMT GR INNER JOIN ENSMIG_FWMigrationNewRules M ON GR.RuleId = M.id


 -- New  GUID generated here
 INSERT INTO dbo.ENSMIG_FWMigrationNewRules(originalRuleId,id)
  SELECT
	  S.id,
	  NEWID() AS id
    FROM HIP8_Rule S
    /*CROSS APPLY
    (
      SELECT  NEWID() AS id
    ) C */
    WHERE S.id NOT IN (SELECT originalRuleId FROM dbo.ENSMIG_FWMigrationNewRules) AND S.schedule_offHours = 'SWITCH'

    --Insert newly created  rules in the following tables using dbo.ENSMIG_FWMigrationNewRules table.

    INSERT INTO FW_RuleMT(id, version, lastModifyingUsername, lastModified, name,note, enabled, action, direction, mediaFlags, tcpFlagsFlags, transportProtocol, localServiceList, remoteServiceList, intrusion, trafficLogged, schedule_start, schedule_end, schedule_offHours, schedule_weekMask, schedule_clickTimeout, DTYPE,leafNodeId, TenantId,schedule_startHours,schedule_startMinutes,schedule_endHours,schedule_endMinutes,scheduleEnabled)
    SELECT
    S2.id,
    S1.version,
    S1.lastModifyingUsername,
    S1.lastModified,
    S1.name+'-reverse',
    S1.note,
    CASE WHEN S1.schedule_offHours!='NONE' AND S1.schedule_weekMask='0' THEN '0' ELSE S1.enabled END,
    CASE WHEN S1.action='ALLOW' THEN 'BLOCK' ELSE 'ALLOW' END,
    S1.direction,
    S1.mediaFlags,
    S1.tcpFlagsFlags,
    S1.transportProtocol,
    S1.localServiceList,
    S1.remoteServiceList,
    S1.intrusion,
    S1.trafficLogged,
    S1.schedule_start,
    S1.schedule_end,
    S1.schedule_offHours,
    S1.schedule_weekMask,
    S1.schedule_clickTimeout,
    S1.DTYPE,
    S1.leafNodeId,
    '1',
    PARSENAME(REPLACE(S1.schedule_start,':','.'),2),
    PARSENAME(REPLACE(S1.schedule_start,':','.'),1),
    PARSENAME(REPLACE(S1.schedule_end,':','.'),2),
    PARSENAME(REPLACE(S1.schedule_end,':','.'),1),
    '0'
	FROM dbo.ENSMIG_FWMigrationNewRules S2
	INNER JOIN HIP8_Rule S1 ON S1.id= S2.originalRuleId

    INSERT INTO dbo.FW_Rule_ApplicationMT(RuleId, ApplicationId, TenantId)
    SELECT
    S3.Id,
    S1.ApplicationId,
    '1'
    FROM HIP8_Rule_Application S1
    INNER JOIN HIP8_Rule S2 ON  S1.RuleId = S2.id
    INNER JOIN ENSMIG_FWMigrationNewRules S3 ON  S1.RuleId = S3.originalRuleId
    WHERE S2.schedule_offHours='SWITCH'

    INSERT INTO dbo.FW_Rule_LocalNetworkMT(RuleId, NamedNetworkId, TenantId)
    SELECT
    S3.Id,
    S1.NamedNetworkId,
    '1'
    FROM HIP8_Rule_LocalNetwork S1
    INNER JOIN HIP8_Rule S2 ON  S1.RuleId = S2.id
    INNER JOIN ENSMIG_FWMigrationNewRules S3 ON  S1.RuleId = S3.originalRuleId
    WHERE S2.schedule_offHours='SWITCH'

    INSERT INTO dbo.FW_Rule_RemoteNetworkMT(RuleId, NamedNetworkId, TenantId)
    SELECT
    S3.Id,
    S1.NamedNetworkId,
    '1'
    FROM HIP8_Rule_RemoteNetwork S1
    INNER JOIN HIP8_Rule S2 ON  S1.RuleId = S2.id
    INNER JOIN ENSMIG_FWMigrationNewRules S3 ON  S1.RuleId = S3.originalRuleId
    WHERE S2.schedule_offHours='SWITCH'

    INSERT INTO dbo.FW_Rule_NetworkProtocolMT(RuleId, networkProtocol, TenantId)
    SELECT
    S3.Id,
    S1.networkProtocol,
    '1'
    FROM HIP8_Rule_NetworkProtocol S1
    INNER JOIN HIP8_Rule S2 ON  S1.RuleId = S2.id
    INNER JOIN ENSMIG_FWMigrationNewRules S3 ON  S1.RuleId = S3.originalRuleId
    WHERE S2.schedule_offHours='SWITCH'

    INSERT INTO dbo.FW_Rule_LocationMT(RuleId, LocationId, TenantId)
    SELECT
    S3.Id,
    S1.LocationId,
    '1'
    FROM HIP8_Rule_Location S1
    INNER JOIN HIP8_Rule S2 ON  S1.RuleId = S2.id
    INNER JOIN ENSMIG_FWMigrationNewRules S3 ON  S1.RuleId = S3.originalRuleId
    WHERE S2.schedule_offHours='SWITCH'


  INSERT INTO ENSMIG_FWMigrationNewGroups(GroupId,ShiftCount) SELECT GROUPID,COUNT(OriginalRuleId) FROM FW_Group_RuleMT GR INNER JOIN ENSMIG_FWMigrationNewRules NR ON GR.RuleId = NR.OriginalRuleId GROUP BY GR.GroupId ORDER BY GR.GroupId
  IF OBJECT_ID('dbo....#mytemp') IS NOT NULL
  DROP TABLE #mytemp

  IF OBJECT_ID('dbo....#mytemp2') IS NOT NULL
  DROP TABLE #mytemp2

  IF OBJECT_ID('dbo....#mytemp3') IS NOT NULL
  DROP TABLE #mytemp3

  select NULL mykey, GR.GroupId,GR.RuleId,GR.ChildIndex,NG.ShiftCount into #mytemp3 from FW_Group_RuleMT GR
  INNER JOIN ENSMIG_FWMigrationNewGroups NG ON GR.GroupId = NG.GroupId
  INNER JOIN ENSMIG_FWMigrationNewRules NR ON GR.RuleId = NR.OriginalRuleId
  ORDER BY GR.GROUPID,ChildIndex DESC
  select * INTO #mytemp FROM #mytemp3
  select DISTINCT GroupId into #mytemp2 from #mytemp3

  Declare @ShiftIndex int
  DECLARE @prev_group_id uniqueidentifier = NEWID()
  DECLARE @current_group_id uniqueidentifier = NEWID();
  set rowcount 1
  update #mytemp3 set mykey = 1
  while @@rowcount > 0
  begin
      set rowcount 0
      select *  into #mytemp8 from #mytemp3 where mykey = 1

    select @current_group_id = GroupId from #mytemp8
    IF(@prev_group_id = @current_group_id)
    BEGIN
        SET @ShiftIndex = @ShiftIndex -1;
    END
    ELSE
    BEGIN
        select @ShiftIndex = ShiftCount from #mytemp8
    END

    UPDATE #mytemp SET ShiftCount = @ShiftIndex FROM #mytemp8 INNER JOIN #mytemp ON #mytemp.GroupId = #mytemp8.GroupId AND #mytemp.RuleId = #mytemp8.RuleId
    set @prev_group_id = @current_group_id

    delete #mytemp3 where mykey = 1

     set rowcount 1
     update #mytemp3 set mykey = 1


      DROP TABLE #mytemp8
  end
  set rowcount 0

  IF OBJECT_ID('dbo....#mytemp5') IS NOT NULL
  DROP TABLE #mytemp5

  select * INTO #mytemp5 FROM  #mytemp ORDER BY GroupId,ChildIndex DESC
  select * FROM #mytemp5

  set rowcount 1
  update #mytemp5 set mykey = 1
  DECLARE @prevChildIndex int
  set @prevChildIndex =-1
  DECLARE @currentChildIndex int
  set @currentChildIndex =-1
  set @prev_group_id = NEWID()
  while @@rowcount > 0
  begin
      set rowcount 0


      select *  into #mytemp6 from #mytemp5 where mykey = 1

    select @currentChildIndex = childIndex from #mytemp6
    select @current_group_id = groupId from #mytemp6
    select @shiftIndex = shiftCount from #mytemp6
    IF(@prev_group_id <> @current_group_id)
    BEGIN
        SET @prevChildIndex = -1;
    END

    IF @prevChildIndex = -1
    BEGIN
      UPDATE dbo.FW_Group_RuleMT SET ChildIndex = ChildIndex + @shiftIndex WHERE GROUPID = @current_group_id AND ChildIndex > @currentChildIndex
    END
    ELSE
    BEGIN
      UPDATE dbo.FW_Group_RuleMT SET ChildIndex = ChildIndex + @shiftIndex WHERE GROUPID = @current_group_id  AND (ChildIndex > @currentChildIndex AND ChildIndex <= @prevChildIndex)
    END



    set @prevChildIndex = @currentChildIndex
    set @prev_group_id = @current_group_id

     delete #mytemp5 where mykey = 1

      set rowcount 1
      update #mytemp5 set mykey = 1

       DROP TABLE #mytemp6
  end

  set rowcount 0



  INSERT INTO dbo.FW_Group_RuleMT (GroupId,RuleId,ChildIndex,TenantId)
  SELECT dbo.FW_Group_RuleMT.GroupId,NR.Id,dbo.FW_Group_RuleMT.ChildIndex+1,TenantId FROM #myTemp
                        INNER JOIN dbo.FW_Group_RuleMT ON #mytemp.GroupId = dbo.FW_Group_RuleMT.GroupId AND #mytemp.RuleId = dbo.FW_Group_RuleMT.RuleId
                        INNER JOIN ENSMIG_FWMigrationNewRules NR ON #mytemp.RuleId = NR.OriginalRuleId

  select * FROM dbo.FW_Group_RuleMT inner join ENSMIG_FWMigrationNewGroups ng  ON dbo.FW_Group_RuleMT.groupid = ng.groupid order by ng.GroupId,childindex

  select * from ENSMIG_FWMigrationNewRules


  select GR.groupid,GR.RuleId,r.name,gr.ChildIndex FROM dbo.FW_Group_RuleMT GR INNER JOIN dbo.FW_RULEMT R ON GR.RuleId = R.id
                                         INNER JOIN ENSMIG_FWMigrationNewGroups NG ON NG.groupId = GR.groupid
                                         ORDER BY GR.GroupId,GR.ChildIndex


  delete from ENSMIG_FWMigrationNewGroups

  END
  GO
